@@ -11,6 +11,10 @@ use cap_primitives::{
     },
     AmbientAuthority,
 };
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
 #[cfg(target_os = "wasi")]
 use posish::fs::OpenOptionsExt;
 use std::{
@@ -582,6 +586,14 @@ impl FromRawFd for Dir {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for Dir {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std_file(fs::File::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawHandle for Dir {
     /// To prevent race conditions on Windows, the handle must be opened without
@@ -592,6 +604,14 @@ impl FromRawHandle for Dir {
     }
 }
 
+#[cfg(windows)]
+impl FromHandle for Dir {
+    #[inline]
+    fn from_handle(handle: OwnedHandle) -> Self {
+        Self::from_std_file(fs::File::from_handle(handle), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for Dir {
     #[inline]
@@ -600,6 +620,14 @@ impl AsRawFd for Dir {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f Dir {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std_file.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandle for Dir {
     #[inline]
@@ -608,6 +636,14 @@ impl AsRawHandle for Dir {
     }
 }
 
+#[cfg(windows)]
+impl<'h> AsHandle<'h> for &'h Dir {
+    #[inline]
+    fn as_handle(self) -> BorrowedHandle<'h> {
+        self.std_file.as_handle()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for Dir {
     #[inline]
@@ -624,6 +660,14 @@ impl IntoRawFd for Dir {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for Dir {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std_file.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandle for Dir {
     #[inline]
@@ -632,6 +676,14 @@ impl IntoRawHandle for Dir {
     }
 }
 
+#[cfg(windows)]
+impl IntoHandle for Dir {
+    #[inline]
+    fn into_handle(self) -> OwnedHandle {
+        self.std_file.into_handle()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for Dir {
     #[inline]
