@@ -2,6 +2,10 @@
 use crate::fs::OpenOptions;
 use crate::fs::{Metadata, Permissions};
 use cap_primitives::{ambient_authority, fs::is_file_read_write, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
 #[cfg(target_os = "wasi")]
 use std::path::Path;
 use std::{
@@ -146,6 +150,14 @@ impl FromRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for File {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(fs::File::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawHandle for File {
     #[inline]
@@ -154,6 +166,14 @@ impl FromRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl FromHandle for File {
+    #[inline]
+    fn from_handle(handle: OwnedHandle) -> Self {
+        Self::from_std(fs::File::from_handle(handle), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for File {
     #[inline]
@@ -162,6 +182,14 @@ impl AsRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f File {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandle for File {
     #[inline]
@@ -170,6 +198,14 @@ impl AsRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl<'h> AsHandle<'h> for &'h File {
+    #[inline]
+    fn as_handle(self) -> BorrowedHandle<'h> {
+        self.std.as_handle()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for File {
     #[inline]
@@ -186,6 +222,14 @@ impl IntoRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for File {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandle for File {
     #[inline]
@@ -194,6 +238,14 @@ impl IntoRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl IntoHandle for File {
+    #[inline]
+    fn into_handle(self) -> OwnedHandle {
+        self.std.into_handle()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for File {
     #[inline]
