@@ -2,6 +2,10 @@
 use crate::fs::OpenOptions;
 use crate::fs::{Metadata, Permissions};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsHandle, BorrowedHandle, FromHandle, IntoHandle, OwnedHandle};
 #[cfg(target_os = "wasi")]
 use std::path::Path;
 use std::{
@@ -126,6 +130,14 @@ impl FromRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for File {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(fs::File::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawHandle for File {
     #[inline]
@@ -134,6 +146,14 @@ impl FromRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl FromHandle for File {
+    #[inline]
+    fn from_handle(handle: OwnedHandle) -> Self {
+        Self::from_std(fs::File::from_handle(handle), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for File {
     #[inline]
@@ -142,6 +162,14 @@ impl AsRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f File {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.cap_std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandle for File {
     #[inline]
@@ -150,6 +178,14 @@ impl AsRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl<'h> AsHandle<'h> for &'h File {
+    #[inline]
+    fn as_handle(self) -> BorrowedHandle<'h> {
+        self.cap_std.as_handle()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for File {
     #[inline]
@@ -166,6 +202,14 @@ impl IntoRawFd for File {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for File {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.cap_std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandle for File {
     #[inline]
@@ -174,6 +218,14 @@ impl IntoRawHandle for File {
     }
 }
 
+#[cfg(windows)]
+impl IntoHandle for File {
+    #[inline]
+    fn into_handle(self) -> OwnedHandle {
+        self.cap_std.into_handle()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for File {
     #[inline]
