@@ -1,5 +1,9 @@
 use crate::net::{Incoming, SocketAddr, TcpStream};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::{fmt, io, net};
 #[cfg(not(windows))]
 use unsafe_io::os::posish::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
@@ -113,6 +117,14 @@ impl FromRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for TcpListener {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::TcpListener::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for TcpListener {
     #[inline]
@@ -124,6 +136,14 @@ impl FromRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for TcpListener {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::TcpListener::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for TcpListener {
     #[inline]
@@ -132,6 +152,14 @@ impl AsRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f TcpListener {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for TcpListener {
     #[inline]
@@ -140,6 +168,14 @@ impl AsRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl<'s> AsSocket<'s> for &'s TcpListener {
+    #[inline]
+    fn as_socket(self) -> BorrowedSocket<'s> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for TcpListener {
     #[inline]
@@ -156,6 +192,14 @@ impl IntoRawFd for TcpListener {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for TcpListener {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for TcpListener {
     #[inline]
@@ -164,6 +208,14 @@ impl IntoRawSocket for TcpListener {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for TcpListener {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for TcpListener {
     #[inline]
