@@ -1,5 +1,9 @@
 use crate::net::{Shutdown, SocketAddr};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::{
     fmt,
     io::{self, IoSlice, IoSliceMut, Read, Write},
@@ -172,6 +176,14 @@ impl FromRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for TcpStream {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::TcpStream::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for TcpStream {
     #[inline]
@@ -180,6 +192,14 @@ impl FromRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for TcpStream {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::TcpStream::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for TcpStream {
     #[inline]
@@ -188,6 +208,14 @@ impl AsRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f TcpStream {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for TcpStream {
     #[inline]
@@ -196,6 +224,14 @@ impl AsRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl<'s> AsSocket<'s> for &'s TcpStream {
+    #[inline]
+    fn as_socket(self) -> BorrowedSocket<'s> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for TcpStream {
     #[inline]
@@ -212,6 +248,14 @@ impl IntoRawFd for TcpStream {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for TcpStream {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for TcpStream {
     #[inline]
@@ -220,6 +264,14 @@ impl IntoRawSocket for TcpStream {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for TcpStream {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for TcpStream {
     #[inline]
