@@ -1,5 +1,9 @@
 use crate::net::{Ipv4Addr, Ipv6Addr, SocketAddr};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+#[cfg(not(windows))]
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
+#[cfg(windows)]
+use io_lifetimes::{AsSocket, BorrowedSocket, FromSocket, IntoSocket, OwnedSocket};
 use std::{fmt, io, net, time::Duration};
 #[cfg(not(windows))]
 use unsafe_io::os::posish::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
@@ -280,6 +284,14 @@ impl FromRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl FromFd for UdpSocket {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(net::UdpSocket::from_fd(fd), ambient_authority())
+    }
+}
+
 #[cfg(windows)]
 impl FromRawSocket for UdpSocket {
     #[inline]
@@ -288,6 +300,14 @@ impl FromRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl FromSocket for UdpSocket {
+    #[inline]
+    fn from_socket(socket: OwnedSocket) -> Self {
+        Self::from_std(net::UdpSocket::from_socket(socket), ambient_authority())
+    }
+}
+
 #[cfg(not(windows))]
 impl AsRawFd for UdpSocket {
     #[inline]
@@ -296,6 +316,14 @@ impl AsRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl<'f> AsFd<'f> for &'f UdpSocket {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std.as_fd()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for UdpSocket {
     #[inline]
@@ -304,6 +332,14 @@ impl AsRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl<'s> AsSocket<'s> for &'s UdpSocket {
+    #[inline]
+    fn as_socket(self) -> BorrowedSocket<'s> {
+        self.std.as_socket()
+    }
+}
+
 #[cfg(windows)]
 impl AsRawHandleOrSocket for UdpSocket {
     #[inline]
@@ -320,6 +356,14 @@ impl IntoRawFd for UdpSocket {
     }
 }
 
+#[cfg(not(windows))]
+impl IntoFd for UdpSocket {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawSocket for UdpSocket {
     #[inline]
@@ -328,6 +372,14 @@ impl IntoRawSocket for UdpSocket {
     }
 }
 
+#[cfg(windows)]
+impl IntoSocket for UdpSocket {
+    #[inline]
+    fn into_socket(self) -> OwnedSocket {
+        self.std.into_socket()
+    }
+}
+
 #[cfg(windows)]
 impl IntoRawHandleOrSocket for UdpSocket {
     #[inline]
