@@ -1,5 +1,6 @@
 use crate::os::unix::net::{Incoming, SocketAddr, UnixStream};
 use cap_primitives::{ambient_authority, AmbientAuthority};
+use io_lifetimes::{AsFd, BorrowedFd, FromFd, IntoFd, OwnedFd};
 use std::{
     fmt, io,
     os::unix::{
@@ -111,6 +112,13 @@ impl FromRawFd for UnixListener {
     }
 }
 
+impl FromFd for UnixListener {
+    #[inline]
+    fn from_fd(fd: OwnedFd) -> Self {
+        Self::from_std(unix::net::UnixListener::from_fd(fd), ambient_authority())
+    }
+}
+
 impl AsRawFd for UnixListener {
     #[inline]
     fn as_raw_fd(&self) -> RawFd {
@@ -118,6 +126,13 @@ impl AsRawFd for UnixListener {
     }
 }
 
+impl<'f> AsFd<'f> for &'f UnixListener {
+    #[inline]
+    fn as_fd(self) -> BorrowedFd<'f> {
+        self.std.as_fd()
+    }
+}
+
 impl IntoRawFd for UnixListener {
     #[inline]
     fn into_raw_fd(self) -> RawFd {
@@ -125,6 +140,13 @@ impl IntoRawFd for UnixListener {
     }
 }
 
+impl IntoFd for UnixListener {
+    #[inline]
+    fn into_fd(self) -> OwnedFd {
+        self.std.into_fd()
+    }
+}
+
 // Safety: `UnixListener` wraps a `net::UnixListener` which owns its handle.
 unsafe impl OwnsRaw for UnixListener {}
 
